//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
//
// ASE.H		
// 	
// ASE API header file for DOS/Win16/Win32
// 
// Include this header file in your sources.
// 
// Copyrights (C) Aladdin Knowledge Systems Ltd.
// 
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#ifndef _ASE_H_
#define _ASE_H_

#ifdef __cplusplus
extern "C" {
#endif

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// OS detection
//
// This file is common to DOS, Windows 3.1x and Windows 95/NT
// The Operating System is detected by macros that are predefiend
// by the compilers Microsoft VC++ and Borland C++
//
// In order the OS detection to work, _NO_OS_DETECT_ should NOT
// be declared.
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#ifdef _NO_OS_DETECT_
	#ifdef _DETECT_OS_
	#undef _DETECT_OS_
	#endif
#else
	#define _DETECT_OS_
#endif

//::::::::::::::::::::::::::::::::::::::::::::::::
//                Microsoft            Borland
//::::::::::::::::::::::::::::::::::::::::::::::::
#ifdef _DETECT_OS_
	#if  (defined(_WIN32)   || defined(__WIN32__))
		#define _ASE_WIN32_
	#endif
	#if ((defined(_WINDOWS) && !defined(_WIN32)) || (defined(_Windows) && !defined(__WIN32__)))
		#define _ASE_WIN16_
	#endif
	#if (defined(_MSDOS)   || defined(__MSDOS__))
		#define _ASE_MSDOS_
	#endif
#endif // DETECT_OS

#if defined(_ASE_WIN32_)
	#include <windows.h>
	#define ASEENTRY __stdcall
#elif defined(_ASE_WIN16_)
	#include <windows.h>
	typedef unsigned char  UCHAR;
	typedef short          SHORT;
	typedef unsigned short USHORT;
	#define ASEENTRY PASCAL
#elif defined(_ASE_MSDOS_)
	#ifndef NULL
		#define NULL    ((void *)0)     
	#endif
	#define FAR      far
	#define ASEENTRY pascal
	typedef unsigned char  UCHAR;
	typedef short          SHORT;
	typedef unsigned short USHORT;
	typedef long           DWORD;
#endif

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// CPU Cards
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#define ASE_CPU_CC1	1
#define ASE_CPU_CP1	2
#define ASE_CPU_CG2	5
#define ASE_CPU_CG3	ASE_CPU_CG2

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// Memory Cards
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#define ASE_MEM_M2  2
#define ASE_MEM_M4  3
#define ASE_MEM_M8  4
#define ASE_MEM_M16 5
#define ASE_MEM_M32 11

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// MP Cards
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#define ASE_MP18 18
#define ASE_MP28 28
#define ASE_MP32 32
#define ASE_MP42 42

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// MPRead() and MPWrite() flags
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
#define ASE_MP_BASE				    (SHORT)0x00
#define ASE_MP_DATA_ONLY 			ASE_MP_BASE+0x00
#define ASE_MP_PROTECTION_ONLY		ASE_MP_BASE+0x01
#define ASE_MP_DATA_AND_PROTECTION  ASE_MP_BASE+0x02
#define ASE_MP_PROTECTED_DATA		ASE_MP_BASE+0x03
#define ASE_MP_VERIFY_DATA			ASE_MP_BASE+0x04
#define ASE_MP_PASSWORD			    ASE_MP_BASE+0x05

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// Common function declarations
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
SHORT FAR ASEENTRY ASE_DRVInit(SHORT, SHORT);
SHORT FAR ASEENTRY ASE_GetVersion(UCHAR FAR *, SHORT);

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// ISO7816 function declarations
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
 
SHORT FAR ASEENTRY ASE_7816CardOn(SHORT,SHORT, UCHAR FAR *);
SHORT FAR ASEENTRY ASE_7816CardOff(void);
SHORT FAR ASEENTRY ASE_7816Receive(SHORT, SHORT, SHORT, SHORT, SHORT, UCHAR FAR *, 
										 UCHAR FAR *, UCHAR FAR *,
										 UCHAR FAR *);
SHORT FAR ASEENTRY ASE_7816Send(SHORT, SHORT, SHORT, SHORT, SHORT, UCHAR FAR *,
									 UCHAR FAR*, UCHAR FAR *,
									 UCHAR FAR*);
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// Memory cards function declarations
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
SHORT FAR ASEENTRY ASE_MCardOn(SHORT);
SHORT FAR ASEENTRY ASE_MCardOff(void);
SHORT FAR ASEENTRY ASE_MDataWrite(SHORT, SHORT, UCHAR FAR*);
SHORT FAR ASEENTRY ASE_MDataRead(SHORT, SHORT, UCHAR FAR*);


//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// Common Cpu function declarations
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
SHORT FAR ASEENTRY ASE_CardOn(SHORT FAR *);
SHORT FAR ASEENTRY ASE_CardOff(void);
SHORT FAR ASEENTRY ASE_FileCreate(SHORT, SHORT, SHORT, SHORT);
SHORT FAR ASEENTRY ASE_FileInit(SHORT, SHORT);
SHORT FAR ASEENTRY ASE_FileWrite(SHORT, SHORT, SHORT, UCHAR FAR*);
SHORT FAR ASEENTRY ASE_FileRead(SHORT, SHORT, SHORT, UCHAR FAR*);
SHORT FAR ASEENTRY ASE_PassChange(SHORT, UCHAR FAR*, UCHAR FAR*);
SHORT FAR ASEENTRY ASE_PassPresent(SHORT, UCHAR FAR*);
SHORT FAR ASEENTRY ASE_PassCreate(SHORT, SHORT,	UCHAR FAR*);

//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// Common Sle function declarations
//:::::::::::::::::::::::::::::::::::::::::::::::::::::::::
SHORT FAR ASEENTRY ASE_MPCardOn(SHORT, UCHAR FAR *);
SHORT FAR ASEENTRY ASE_MPCardOff(void);
SHORT FAR ASEENTRY ASE_MPWrite(SHORT, UCHAR, SHORT);
SHORT FAR ASEENTRY ASE_MPRead(SHORT, SHORT, UCHAR FAR *, UCHAR FAR *, SHORT);
SHORT FAR ASEENTRY ASE_MPVerifyPassword(UCHAR FAR *);

#ifdef __cplusplus
}
#endif

#endif // _ASE_H_
