/**************************************************************************
 * SMARTCARD NATIVE API
 * (c) Iman Budi Setiawan, 1999
 * ------------------------------------------------------------------------
 * File ini menghasilkan CPUCard.dll yang merupakan API untuk menghubungkan
 * ASE dengan Java.
 * Ket : link dengan Ase32.lib untuk versi Win32.
 **************************************************************************/

#include <string.h>
#include "ase.h"
#include "SmartCard.h"


/*------------------------------------------------------------------------
 * InitDrive ()
 *
 * Search for the ASEDrive port (0 = auto search).
 * Set the TimeOut response value (5 seconds).
 *------------------------------------------------------------------------*/
SHORT InitDrive() {
	SHORT port;
	SHORT error;

	// set mode port ke PC auto-detect.
	port = 0;

	error = ASE_DRVInit ( port, 5 );

	// jika error=0 maka ASEDrive berhasil ditemukan
	// selain itu terjadi kesalahan.
	return error;
}

/*------------------------------------------------------------------------
 * TurnOn ()
 *
 * Initialize the ASEDrive : Turn the ASEDrive's power on.
 *	                       	   Reset the card.
 *------------------------------------------------------------------------*/
SHORT TurnOn() {
	SHORT error;
	USHORT CardType = 1;

	error = ASE_CardOn ( (USHORT FAR *) &CardType );
	
	// jika error=0 berarti power card sudah on
	// selain itu masih off
	return error;
}

/*------------------------------------------------------------------------
 * TurnOff
 *
 * Terminate the work with the ASEDrive.
 *------------------------------------------------------------------------*/
SHORT TurnOff() {
	SHORT error;

	error = ASE_CardOff ();

	// jika error=0 berarti termination berhasil
	return error;
}

/*------------------------------------------------------------------------
 * FileCreate ()
 *
 * 1. Create a new file:  filename 10, file size 40 bytes.
 * 2. Password number 1 enables reading from and writing to 
 * 	the file.
 *------------------------------------------------------------------------*/
SHORT FileCreate(SHORT FileNumber, SHORT FileSize, SHORT ReadPassIndex, SHORT WritePassIndex) {
	SHORT error;

	error = ASE_FileCreate ( FileNumber, FileSize, ReadPassIndex, WritePassIndex);
	
	// jika error=0 berarti berhasil buat file
	return error;
}

/*------------------------------------------------------------------------
 * CreatePassword ()
 *
 * Written the password to the card.
 *------------------------------------------------------------------------*/
SHORT CreatePassword(SHORT Index, SHORT Attemp, CONST CHAR FAR* Password) {
	SHORT error;

	error = ASE_PassCreate ( Index, Attemp, (UCHAR FAR *)Password );
	return error;
}

/*------------------------------------------------------------------------
 * VerifyPassword ()
 *
 * Present the password that was just written.
 *------------------------------------------------------------------------*/
SHORT VerifyPassword(SHORT Index, CONST CHAR FAR* Password) {
	SHORT error;

	error = ASE_PassPresent ( Index, (UCHAR FAR *)Password );

	// jika error=0 berarti password valid
	return error;
}

/*------------------------------------------------------------------------
 * Load ()
 *
 * Baca data yang sudah ditulis sebelumnya.
 *------------------------------------------------------------------------*/
CHAR* Load(SHORT FileNumber, SHORT Offset, SHORT FileSize) {
	SHORT error;
	char* data = "";

	// baca file di SmartCard
	error = ASE_FileRead ( FileNumber, Offset, FileSize, data );
	if (strlen(data) > FileSize-Offset) data[FileSize-Offset] = 0;

	// jika error=0 berarti berhasil membaca file
	if (error!=0) return "";
	return data;
}

/*------------------------------------------------------------------------
 * Store ()
 *
 * Simpan data ke file di SmartCard
 *------------------------------------------------------------------------*/
SHORT Store(SHORT FileNumber, SHORT Offset, SHORT FileSize, CONST CHAR FAR* data) {
	SHORT error;

	// Initialize the file that has just been created.
	error = ASE_FileInit ( FileNumber, FileSize );

	// jika file belum ada maka keluar fungsi
	if ( error != 0 ) return error;


	// Write the data "abcd" to file 10, at offset 20.
	error = ASE_FileWrite ( FileNumber, Offset, FileSize, (UCHAR FAR*) data );

	// jika error=0 berarti data berhasil ditulis
	return error;

}

/*------------------------------------------------------------------------
 * Class:     SmartCard
 * Method:    LoadChar
 * Signature: (SSS)[C
 *------------------------------------------------------------------------*/
JNIEXPORT jcharArray JNICALL Java_SmartCard_LoadChar
  (JNIEnv *env, jobject obj, jshort FileNumber, jshort Offset, jshort FileSize) {
	// buka file di smart card disimpan dalam data dan ubah data ke string
	jcharArray jca = Load(FileNumber, Offset, FileSize);
	return jca;
}

/*------------------------------------------------------------------------
 * Class:     SmartCard
 * Method:    StoreChar
 * Signature: (SSS[C)S
 *------------------------------------------------------------------------*/
JNIEXPORT jshort JNICALL Java_SmartCard_StoreChar
  (JNIEnv *env, jobject obj, jshort FileNumber, jshort Offset, jshort FileSize, jcharArray chrdata) {
	// inisialisasi
	jshort error;
	// panggil fungsi store
	error = Store(FileNumber, Offset, FileSize, chrdata);
	return error;

}

/*------------------------------------------------------------------------
 * Class:     SmartCard
 * Method:    CreatePassword
 * Signature: (SSLjava/lang/String;)S
 *------------------------------------------------------------------------*/
JNIEXPORT jshort JNICALL Java_SmartCard_CreatePassword
  (JNIEnv *env, jobject obj, jshort Index, jshort Attemp, jstring strPassword) {
	// inisialisasi
	jshort error;
	const char* Password = (*env)->GetStringUTFChars(env, strPassword, 0);
	// panggil fungsi create password
	error = CreatePassword(Index, Attemp, Password);
	// dealokasi UTF char
	(*env)->ReleaseStringUTFChars(env, strPassword, Password);
	return error;
}

/*------------------------------------------------------------------------
 * Class:     SmartCard
 * Method:    FileCreate
 * Signature: (SSSS)S
 *------------------------------------------------------------------------*/
JNIEXPORT jshort JNICALL Java_SmartCard_CreateFile
  (JNIEnv *env, jobject obj, jshort FileNumber, jshort FileSize, jshort ReadPassIndex, jshort WritePassIndex) {
	return FileCreate(FileNumber, FileSize, ReadPassIndex, WritePassIndex);
}

/*------------------------------------------------------------------------
 * Class:     SmartCard
 * Method:    FileRead
 * Signature: (SSSLjava/lang/String;)S
 *------------------------------------------------------------------------*/
JNIEXPORT jstring JNICALL Java_SmartCard_Load
  (JNIEnv *env, jobject obj, jshort FileNumber, jshort Offset, jshort FileSize) {
	// buka file di smart card disimpan dalam data dan ubah data ke string
	jstring strdata = (*env)->NewStringUTF (env, (CONST CHAR*)Load(FileNumber, Offset, FileSize));
	return strdata;
}

/*------------------------------------------------------------------------
 * Class:     SmartCard
 * Method:    FileWrite
 * Signature: (SSSLjava/lang/String;)S
 *------------------------------------------------------------------------*/
JNIEXPORT jshort JNICALL Java_SmartCard_Store
  (JNIEnv *env, jobject obj, jshort FileNumber, jshort Offset, jshort FileSize, jstring strdata) {
	// inisialisasi
	jshort error;
	const char* data = (*env)->GetStringUTFChars(env, strdata, 0);
	// panggil fungsi store
	error = Store(FileNumber, Offset, FileSize, data);
	// dealokasi UTF char
	(*env)->ReleaseStringUTFChars(env, strdata, data);
	return error;
}

/*------------------------------------------------------------------------
 * Class:     SmartCard
 * Method:    InitDrive
 * Signature: ()S
 *------------------------------------------------------------------------*/
JNIEXPORT jshort JNICALL Java_SmartCard_InitDrive
  (JNIEnv *env, jobject obj) {
	return InitDrive();
}

/*------------------------------------------------------------------------
 * Class:     SmartCard
 * Method:    TurnOff
 * Signature: ()S
 *------------------------------------------------------------------------*/
JNIEXPORT jshort JNICALL Java_SmartCard_TurnOff
  (JNIEnv *env, jobject obj) {
	return TurnOff();
}

/*------------------------------------------------------------------------
 * Class:     SmartCard
 * Method:    TurnOn
 * Signature: ()S
 *------------------------------------------------------------------------*/
JNIEXPORT jshort JNICALL Java_SmartCard_TurnOn
  (JNIEnv *env, jobject obj) {
	return TurnOn();
}

/*------------------------------------------------------------------------
 * Class:     SmartCard
 * Method:    VerifyPassword
 * Signature: (SLjava/lang/String;)S
 *------------------------------------------------------------------------*/
JNIEXPORT jshort JNICALL Java_SmartCard_VerifyPassword
  (JNIEnv *env, jobject obj, jshort Index, jstring strPassword) {
	// inisialisasi
	jshort error;
	const char* Password = (*env)->GetStringUTFChars(env, strPassword, 0);
	// panggil fungsi verify password
	error = VerifyPassword(Index, Password);
	// dealokasi UTF char
	(*env)->ReleaseStringUTFChars(env, strPassword, Password);
	return error;
}
